/****************************************************************************************
	
	Ermittlung der Jordanschen Normalform einer komplexen Matrix (mit 
	verschwindendem Imaginrteil) ber den komplexen Zahlen
	------ in ANSI-C ----------
	
	Vorgelegt im Rahmen des NUMERIKPRAKTIKUMs von 
	Sascha Richter, MatrNr. 9980878, Sommersemester 2007, Universitt Leipzig
	Betreuer: Dr. Stefan Ackermann
	
*****************************************************************************************
	Quelle fr einige Algorithmen:
		Press, W. et al.: Numerical Recipes in C, Cambridge University Press 1992
		(online: http://www.nrbook.com/a/bookcpdf.php)

*****************************************************************************************
	Entwickelt mit
		Compiler lcc-win32, Version 3.8
		Linker   lcclnk,    Version 1.3

*******************************************************************************************/

#include <stdio.h> 		// fprintf(), printf(), fopen(), fclose(), fgets(),
						// stderr(), FILE
						// Fuer die Kernfunktionalitaet des Progamms ist die
						// stdio.h entbehrlich.
#include <stdlib.h>		// malloc(), free(), exit()
#include <math.h>		// fabs(), sqrt()
#include <string.h>		// strtok(), atof()
						// Fuer die Kernfunktionalitaet des Progamms ist die
						// string.h entbehrlich.
#include "mymathlib.c"	// http://mymathlib.webtrellis.net/matrices.html


#define EPSILON 0.00001  // isnullc(), gleich(), alg_vielf()
#define BASETYPE double // Der Datentyp, mit dem alle floating-point- 
						// Berechnungen	ausgefhrt werden. Eine Aenderung hier
						// erfordert ggf. Anpassungen bei den Formatierungs-
						// anweisungen fr printf() und bei EPSILON.

#define LCC32	// auskommentieren, wenn man nicht den LCC32-Compiler verwendet
				// MAT, VEC, MNULL
				// m_get(),v_get(),m_copy(),schur(),schur_evals()
#ifdef LCC32
	#include "matrix.h"		// von LCC
#endif




// Datentyp zur Darstellung komplexer Zahlen
		typedef struct 
		{
			BASETYPE re;
			BASETYPE im;
		} KOMPLEX;

/* IO-Funktionen */
		// Anzahl der Zeilen in einer ASCII-Datei
		int  getlines(char *dat);
		// Wie der Name schon sagt...
		void datei2matrix(BASETYPE **x,char *dat,int dim);

	
/* Algorithmen aus "Numerical Recipes in C", Kapitel 11.5 und 11.6 */
		#define RADIX 2.0  //Fr balanc; Basis, zu der die Maschine rechnet
		// Eine (optionale) Vorbereitung fr elmhes
		void balanc(BASETYPE **a, int n);
		#define SWAP(g,h) {y=(g);(g)=(h);(h)=y;} //Fr elmhes
		// Bringt die Matrix a auf obere Hessenbergform
		void elmhes(BASETYPE **a, int n);
		// Zur Berechnung der Eigenwerte: QR-Zerlegung einer oberen Hessenbergmatrix
		// Real- bzw. Imaginrteile der Eigenwerte werden in wr bzw. wi abgelegt
		void hqr (BASETYPE **a, int n, BASETYPE wr[], BASETYPE wi[]);

/* selbstgebaute numerische Funktionen */
		// gibt die Summe 2er komplexen Zahlen zurck
		KOMPLEX addiere(KOMPLEX a,KOMPLEX b);
		// gibt das Produkt 2er komplexen Zahlen zurck
		KOMPLEX multipl(KOMPLEX a,KOMPLEX b);
		//Betragsquadrat einer komplexen Zahl
		BASETYPE betragsq(KOMPLEX a); 
		// tauscht u-te und v-te Zeile einer komplexen nxn-Matrix y in situ
		void rowswap(KOMPLEX **y,int u,int v,int n);
		// tauscht u-te und v-te Spalte einer komplexen nxn-Matrix y in situ
		void colswap(KOMPLEX **y,int u,int v,int n);
		//checkt, ob a == 0+0I
		int isnullc(KOMPLEX a);	
		// Gauss-Algorithmus fr eine komplexe nxn-Matrix a in situ
		void gaussc(KOMPLEX **a,int n); 
		// Ermittelt den Rang einer komplexen Matrix	
		int rankc(KOMPLEX **a,int n);
		// Produkt 2er quadratischer nxn-Matrizen a,b mit komplexen Eintrgen
		// Ergebnis kommt nach c
		void mat_mul(KOMPLEX **a,KOMPLEX **b,KOMPLEX **c, int n);
		// k-te (0,1,2,..) Potenz einer nxn-Matrix a, Ergebnis kommt nach b 
		void mat_pot(KOMPLEX **a, KOMPLEX **b,int k, int n);
		// Berechnet den Rang von (a-lbda*E)^k
		// Dabei ist a eine komplexe nxn-Matrix und lbda*E=diag(lbda,lbda,...)
		int rg_sub_pot(KOMPLEX **a,KOMPLEX lbda,int k,int n);
		// berprft, ob 2 komplexe Zahlen z,u "rund gleich" sind
		// von z wird Re und Im separat bergeben
		int gleich(BASETYPE re,BASETYPE im, KOMPLEX u);
		// Identifiziert gleiche Eigenwerte und ermittelt die algebraischen Vielfachheiten
		// Returnwert: Anzahl der verschiedenen Eigenwerte
		int alg_vielf(	BASETYPE *wr,// Real- und
						BASETYPE *wi,// Imaginrteile der von hqr ermittelten Eigenwerte
						KOMPLEX *ev, // der doublettenfreie Vektor der Eigenwerte
						int n,
						int *av);	 // Vektor der algebraischen Vielfachheiten in der 
									 // Reihenfolge entsprechend av
 	    // 	Ermittelt die Anzahlen k[i,j], mit der die Jordankstchen J(lbda(i),j) in
		//	der JNF vorkommen
		void jblocks(	KOMPLEX **a,// die Ausgangsmatrix
						KOMPLEX *ew,// Vektor der (verschiedenen) Eigenwerte
						int *av,	// Vektor der zugehrigen algebraischen Vielfachheiten
						int r,		// Anzahl der verschiedenen Eigenwerte
						int n,		// Dimension von a
						int **kij);	// Matrix der zu berechnenden k[i,j]
		// Ermittelt die JNF jf der Matrix a
		void jnf(KOMPLEX **a,KOMPLEX **jf,int n);







	/* Testprogramm */
int main(int argc, char **argv)
{
		if(argc < 2)
			{printf("Verwendung: %s MATRIX-DATEI\n",argv[0]);
			 exit(0);}
		// Ermittlung der Zeilenzahl der Matrix
		int n=getlines(argv[1]);
		int i,j;
		// Matrix, in die aus der Datei eingelesen wird
		BASETYPE **x=(BASETYPE **)malloc((n+1)*sizeof(BASETYPE *));
			for(i=1;i<=n;i++) x[i]=(BASETYPE *)malloc((n+1)*sizeof(BASETYPE));
		datei2matrix(x,argv[1],n);
		// Die Eingabematrix liegt jetzt in x


		// Ausgabe der Eingabematrix
		printf("\nEingabematrix:\n");
		for(i=1;i<=n;i++)
			for(j=1;j<=n;j++)
			{
				printf(" %+9.2f ",x[i][j]);
				if(j%n==0)printf("\n");
			}

		
		// Die Funktion "jnf" erwartet (fr knftige Programmversionen) eine
		// echt komplexe Matrix als Argument; also wird die Ausgangsmatrix 
		// "komplex gemacht"
		KOMPLEX **a=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
			for(i=1;i<=n;i++) a[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
		
		for(i=1;i<=n;i++)
			for(j=1;j<=n;j++)	
				{	a[i][j].im=0.;
					a[i][j].re=x[i][j];	}
		
		// Deklaration der JNF
		KOMPLEX **jf=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
			for(i=1;i<=n;i++) jf[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
		// Berechnung der JNF der Matrix a	
		jnf(a,jf,n);
	
		// Ausgabe der JNF
		printf("\nJNF (Haupt- und Superdiagonale):\n");
		for(i=1;i<=n;i++)
			for(j=1;j<=n;j++)
			{
				/*if(j==i)
					printf("%+7.3f%+7.3fI",jf[i][j].re,jf[i][j].im);
				else if(j==i+1)
					printf("      %d",(int)jf[i][j].re);
				else printf("               ");
				if(j%n==0)printf("\n");
				*/
				if(j==i)
					printf("%+7.3f%+7.3fI\n",jf[i][j].re,jf[i][j].im);
				else if(j==i+1)
					printf("                  %d\n",(int)jf[i][j].re);

			}
			
	return 0;
}




/*Numerical Recipes in C Kapitel 11.5
"Given a matrix a[1..n][1..n], this routine replaces it by a balanced matrix with identical
eigenvalues. A symmetric matrix is already balanced and is unaffected by this procedure. The
parameter RADIX should be the machines floating-point radix."*/
void balanc(BASETYPE **a, int n)
{
	int last,j,i;
	BASETYPE s,r,g,f,c,sqrdx;
	sqrdx=RADIX*RADIX;
	last=0;
	while (last == 0) 
	{
		last=1;
		for (i=1;i<=n;i++) 
		{ 
			r=c=0.0;
			for (j=1;j<=n;j++)
			if (j != i) 
			{
				c += fabs(a[j][i]);
				r += fabs(a[i][j]);
			}
			if (c && r) 
			{ 
				g=r/RADIX;
				f=1.0;
				s=c+r;
				while (c<g) 
				{ 
					f *= RADIX; 
					c *= sqrdx;
				}
				g=r*RADIX;
				while (c>g) 
				{
					f /= RADIX;
					c /= sqrdx;
				}
				if ((c+r)/f < 0.95*s) 
				{
					last=0;
					g=1.0/f;
					for (j=1;j<=n;j++) a[i][j] *= g; 
					for(j=1;j<=n;j++) a[j][i] *= f; 
				}
			}
		}
	}
}




/*Numerical Recipes in C Kapitel 11.5
Bringt die Matrix a auf obere Hessenberg-Form
(a wird damit berschrieben) 
"Reduction to Hessenberg form by the elimination method. The real, nonsymmetric matrix
a[1..n][1..n] is replaced by an upper Hessenberg matrix with identical eigenvalues. Recommended,
but not required, is that this routine be preceded by balanc. On output, the
Hessenberg matrix is in elements a[i][j] with i <= j+1. Elements with i > j+1 are to be
thought of as zero, but are returned with random values."*/
void elmhes(BASETYPE **a, int n)
{
	int m,j,i;
	BASETYPE y,x;
	for (m=2;m<n;m++) 
	{ 
		x=0.0;
		i=m;
		for (j=m;j<=n;j++) 
		{ 
			if (fabs(a[j][m-1]) > fabs(x)) 
			{
				x=a[j][m-1];
				i=j;
			}
		}
		if (i != m) 
		{ 
			for (j=m-1;j<=n;j++) SWAP(a[i][j],a[m][j])
			for (j=1;j<=n;j++) SWAP(a[j][i],a[j][m])
		}
		if (x) 
		{ 
			for (i=m+1;i<=n;i++) 
			{
				if ((y=a[i][m-1]) != 0.0) 
				{
					y /= x;
					a[i][m-1]=y;
					for (j=m;j<=n;j++)
					a[i][j] -= y*a[m][j];
					for (j=1;j<=n;j++)
					a[j][m] += y*a[j][i];
				}
			}
		}
	}
}






/*Numerical Recipes in C Kapitel 11.6
Fhrt eine QR-Zerlegung fr eine Matrix a in oberer Hessenberg-Form 
durch; Die Real- bzw. Imaginrteile der Eigenwerte von a werden 
in wr bzw. wi abgelegt
"Finds all eigenvalues of an upper Hessenberg matrix a[1..n][1..n]. On input a can be
exactly as output from elmhes 11.5; on output it is destroyed. The real and imaginary parts
of the eigenvalues are returned in wr[1..n] and wi[1..n], respectively."
*/
void hqr(BASETYPE **a, int n, BASETYPE wr[], BASETYPE wi[])
{
	int nn,m,l,k,j,its,i,mmin;
	BASETYPE z,y,x,w,v,u,t,s,r,q,p,anorm;
	anorm=0.0; 
	// oder 
	anorm = fabs (a[1][1]);
	/*
	so zB in
		http://info.math.nankai.edu.cn/mathdoc/Computing/Numerical%2520
		Recipes/Numerical%2520Recipes%2520Source-%2520C2.02.zip&cs_f=nr-2.02/
		c/kr/recipe/hqr.c&start=1	
	*/
	for(i=1;i<=n;i++)
		for (j=(((i-1>1)?(i-1):1));j<=n;j++)
			anorm += fabs(a[i][j]);
	nn=n;
	t=0.0; 
	while (nn >= 1) 
		{ 
		its=0;
		do 
		{
			for (l=nn;l>=2;l--) 
			{ 
				s=fabs(a[l-1][l-1])+fabs(a[l][l]); 
				if (s == 0.0) s=anorm;
				if ((BASETYPE)(fabs(a[l][l-1]) + s) == s) 
				{
					//a[l][l-1]=0.0;
					break;
				}
			}
			x=a[nn][nn];
			if (l == nn) 
			{ //One root found.
				wr[nn]=x+t;
				wi[nn--]=0.0;
			} else 
			{
				y=a[nn-1][nn-1];
				w=a[nn][nn-1]*a[nn-1][nn];
				if (l == (nn-1)) 
				{// Two roots found...
					p=0.5*(y-x);
					q=p*p+w;
					z=sqrt(fabs(q));
					x += t;
					if (q >= 0.0) 
					{ //...a real pair.
						z=p+((p>0.0)?fabs(z) : -fabs(p));
						wr[nn-1]=wr[nn]=x+z;
						if (z) wr[nn]=x-w/z;
						wi[nn-1]=wi[nn]=0.0;
					} else 
					{ //...a complex pair.
						wr[nn-1]=wr[nn]=x+p;
						wi[nn-1]= -(wi[nn]=z);
					}
					nn -= 2;
				} 
				else //No roots found. Continue iteration.
				{ 
					if (its == 30) printf("Too many iterations in hqr");
					if (its == 10 || its == 20) //Form exceptional shift.
					{ 
						t += x;
						for (i=1;i<=nn;i++) a[i][i] -= x;
						s=fabs(a[nn][nn-1])+fabs(a[nn-1][nn-2]);
						y=x=0.75*s;
						w = -0.4375*s*s;
					}
					++its;
					for (m=(nn-2);m>=l;m--) //Form shift and then look for
					{ 						//2 consecutive small subdiagonal elements.
						z=a[m][m];
						r=x-z;
						s=y-z;
						p=(r*s-w)/a[m+1][m]+a[m][m+1]; //Equation (11.6.23).
						q=a[m+1][m+1]-z-r-s;
						r=a[m+2][m+1];
						s=fabs(p)+fabs(q)+fabs(r); //Scale to prevent overflow or
						p /= s; //underflow.
						q /= s;
						r /= s;
						if (m == l) break;
						u=fabs(a[m][m-1])*(fabs(q)+fabs(r));
						v=fabs(p)*(fabs(a[m-1][m-1])+fabs(z)+fabs(a[m+1][m+1]));
						if ((BASETYPE)(u+v) == v) break; //Equation (11.6.26).
					}
					for (i=m+2;i<=nn;i++) 
					{
						a[i][i-2]=0.0;
						if (i != (m+2)) a[i][i-3]=0.0;
					}
					for (k=m;k<=nn-1;k++) 
					{
						//Double QR step on rows l to nn and columns m to nn.
						if (k != m) 
						{
							p=a[k][k-1]; //Begin setup of Householder vector.
							q=a[k+1][k-1]; 
							r=0.0;
							if (k != (nn-1)) r=a[k+2][k-1];
							if ((x=fabs(p)+fabs(q)+fabs(r)) != 0.0) 
							{
								p /= x; //Scale to prevent overflow or underflow.
								q /= x; 
								r /= x;
							}
						}
						if ((s=(((p) >= 0.0) ? fabs(sqrt(p*p+q*q+r*r)) : -fabs(sqrt(p*p+q*q+r*r)))) != 0.0)
						{
							if (k == m) 
							{
								if (l != m)
								a[k][k-1] = -a[k][k-1];
							} else
							a[k][k-1] = -s*x;
							p += s; //Equations (11.6.24).
							x=p/s;
							y=q/s;
							z=r/s;
							q /= p;
							r /= p;
							for (j=k;j<=nn;j++) //Row modification.
							{ 
								p=a[k][j]+q*a[k+1][j];
								if (k != (nn-1)) 
								{
									p += r*a[k+2][j];
									a[k+2][j] -= p*z;
								}
								a[k+1][j] -= p*y;
								a[k][j] -= p*x;
							}
							mmin = nn<k+3 ? nn : k+3;
							for (i=l;i<=mmin;i++) //Column modification.
							{ 
								p=x*a[i][k]+y*a[i][k+1];
								if (k != (nn-1)) 
								{
									p += z*a[i][k+2];
									a[i][k+2] -= p*r;
								}
								a[i][k+1] -= p*q;
								a[i][k] -= p;
							}
						}
					}
				}
			}
		} while (l < nn-1);
	}
}







/* IO-Funktionen *************************************
******************************************************/
/* Liest eine Matrix aus einer Datei und schreibt sie in das Array x
	Beispiel fr eine Matrix-Datei:
	1,1,1,
	1.1,2,3,
	-4,-4,-4
	Unter Verwendung einer Idee von Wolf, Jrgen: C von A bis Z,
	http://www.pronix.de/pronix-737.html*/
void datei2matrix(BASETYPE **x,char *dat,int n)   
{  	FILE *datei;		
	int i,j;
	char line[350],*ptr;
	datei=fopen(dat,"r");
	if(NULL==datei)
		{printf("Fehler beim Dateioeffnen.");exit(0);}
	for	(i=1;i<=n;i++)
	{
		fgets(line,350,datei);
		ptr = strtok(line, ",");
		for(j=1;j<=n;j++)
			{
			 x[i][j]=atof(ptr);
			 ptr = strtok(NULL, ",");
			 }
	}	
	fclose(datei);
}

/*Ermittelt die Anzahl der Zeilen in einer ASCII-Datei*/
int getlines(char *dat)
{	
	FILE *datei;
	char line[400];
	int i=0;
	datei=fopen(dat,"r");
	if(NULL == datei) {fprintf(stderr, "Fehler beim Oeffnen ...\n");
					   return -1;}
		while(fgets(line,400,datei)!=NULL)   i++;
	fclose(datei);
	return i;
}


/*************************************************************/

// Ermittelt den Rang der komplexen nxn-Matrix a
int rankc(KOMPLEX **a,int n)
{
	int i,j,r=n;
	KOMPLEX **x=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
		for(i=1;i<=n;i++) x[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
	for(i=1;i<=n;i++)
		for(j=1;j<=n;j++)
			{x[i][j].re=a[i][j].re;
			 x[i][j].im=a[i][j].im;}
	gaussc(x,n);
	i=n;
	while(isnullc(x[i][n]) && i>=1)
		{i--;
		 r--;}
	for(i=1;i<=n;i++)
		free(x[i]);
	free(x);
	return r;
}

KOMPLEX addiere(KOMPLEX a,KOMPLEX b)
{
	KOMPLEX c;
	c.re=a.re+b.re;
	c.im=a.im+b.im;
	return c;
}

KOMPLEX multipl(KOMPLEX a,KOMPLEX b)
{
	KOMPLEX c;
	c.re=a.re*b.re-a.im*b.im;
	c.im=a.re*b.im+a.im*b.re;
	return c;
}

//Betragsquadrat von a
BASETYPE betragsq(KOMPLEX a) 
{
	return ((a.re*a.re+a.im*a.im));
}

// tauscht u-te und v-te Zeile einer komplexen nxn-Matrix y in situ
void rowswap(KOMPLEX **y,int u,int v,int n)
{
	KOMPLEX tmp[n+1];
	int i;
	for (i=1;i<=n;i++)
		{ tmp[i]=y[u][i];
		 y[u][i]=y[v][i];
		 y[v][i]=tmp[i];
		}
}

// tauscht u-te und v-te Spalte einer komplexen nxn-Matrix y in situ
void colswap(KOMPLEX **y,int u,int v,int n)
{
	KOMPLEX tmp[n+1];
	int i;
	for (i=1;i<=n;i++)
		{ tmp[i]=y[i][u];
		 y[i][u]=y[i][v];
		 y[i][v]=tmp[i];
		}
}


//checkt, ob a == 0
int isnullc(KOMPLEX a)	
{	
	if(betragsq(a)<EPSILON) return 1;// TRUE
	else return 0; 			      // FALSE
}


// Gauss-Algorithmus fr eine komplexe nxn-Matrix a in situ
// bringt a in Dreiecks- bzw. Stufenform
void gaussc(KOMPLEX **a,int n) 
{
	int i,j,k;
	int q,r,u,v;
	KOMPLEX faktor;
	BASETYPE tmp;
	
	for(i=1;i<n;i++)
	{
		// Suche (Spalten-)Pivot
		//	for(k=i+1;k<=n;k++)
		//		if(betragsq(a[k][i])>betragsq(a[i][i])) rowswap(a,i,k,n);
		
		//Suche totales Pivot
		u=0;v=0;
		tmp=betragsq(a[i][i]);
		for(q=i;q<=n;q++)
			for(r=i;r<=n;r++)
				if(betragsq(a[q][r])>tmp)
					{	
						tmp=betragsq(a[q][r]);
						u=q;
						v=r;
					}
		if(u>0) rowswap(a,i,u,n);
		if(v>0) colswap(a,i,v,n);
				
		
		if(!isnullc(a[i][i]))
		{		
			for(k=i+1;k<=n;k++)
				if(!isnullc(a[k][i]))
				{
					faktor.re=(-a[i][i].re*a[k][i].re-a[i][i].im*a[k][i].im)/(a[i][i].re*a[i][i].re+a[i][i].im*a[i][i].im);
					faktor.im=( a[i][i].im*a[k][i].re-a[i][i].re*a[k][i].im)/(a[i][i].re*a[i][i].re+a[i][i].im*a[i][i].im);
					a[k][i].re=0.;
					a[k][i].im=0.;			
					for(j=i+1;j<=n;j++)
						a[k][j]=addiere(a[k][j],multipl(faktor,a[i][j]));
				}
			
		}
	}
}



// Produkt 2er quadratischer nxn-Matrizen a,b mit komplexen Eintrgen
// Ergebnis kommt nach c
void mat_mul(KOMPLEX **a,KOMPLEX **b,KOMPLEX **c, int n)
{
	int i,j,l;
	for(i=1;i<=n;i++)
		for(j=1;j<=n;j++)
		{
			c[i][j].re=0.;//Man weiss ja nie...
			c[i][j].im=0.;
			for(l=1;l<=n;l++)
				c[i][j]=addiere(c[i][j],multipl(a[i][l],b[l][j]));
		}
}



/* k-te (0,1,2,..) Potenz einer nxn-Matrix a, Ergebnis kommt nach b */
void mat_pot(KOMPLEX **a, KOMPLEX **b,int k, int n)
{
	int i,j,m;
	if(k==0)
		for(i=1;i<=n;i++)
			for(j=1;j<=n;j++)
				{if(i==j) 	{b[i][j].re=1.;b[i][j].im=0.;}
				 else 		{b[i][j].re=0.;b[i][j].im=0.;}
				}
	if(k==1)
		for(i=1;i<=n;i++)
			for(j=1;j<=n;j++)
				b[i][j]=a[i][j];
	if(k>=2)
		{
			KOMPLEX **t1=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
				for(i=1;i<=n;i++) t1[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
			KOMPLEX **t2=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
				for(i=1;i<=n;i++) t2[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
			for(i=1;i<=n;i++)
				for(j=1;j<=n;j++)
					t1[i][j]=a[i][j];
			for(m=2;m<=k;m++)
				{
					mat_mul(t1,a,t2,n);
					for(i=1;i<=n;i++)
						for(j=1;j<=n;j++)
							t1[i][j]=t2[i][j];
				}
			for(i=1;i<=n;i++)
				for(j=1;j<=n;j++)
					b[i][j]=t2[i][j];

			//http://www.pronix.de/pronix-762.html:
			for(i=1;i<=n;i++)
				{free(t1[i]);
				 free(t2[i]);}
			free(t1);free(t2);
		}
}

// Berechnet den Rang von (a-lbda*E)^k
// Dabei ist a eine komplexe nxn-Matrix und lbda*E=diag(lbda,lbda,...)
int rg_sub_pot(KOMPLEX **a,KOMPLEX lbda,int k,int n)
{
	int i,j;
	KOMPLEX **x=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
		for(i=1;i<=n;i++) x[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
	for(i=1;i<=n;i++)
		for(j=1;j<=n;j++)
			{x[i][j].re=a[i][j].re;
			 x[i][j].im=a[i][j].im;}
	lbda.re=-lbda.re;
	lbda.im=-lbda.im;
	for(i=1;i<=n;i++)
		x[i][i]=addiere(a[i][i],lbda);
	KOMPLEX **y=(KOMPLEX **)malloc((n+1)*sizeof(KOMPLEX *));
		for(i=1;i<=n;i++) y[i]=(KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));
	mat_pot(x,y,k,n);
	int r=rankc(y,n);

	for(i=1;i<=n;i++)
		{free(x[i]);free(y[i]);}
	free(x);free(y);
	return r;
}









// berprft, ob 2 komplexe Zahlen z,u "rund gleich" sind
// Re(z) und Im(z) werden separat bergeben
int gleich(BASETYPE re,BASETYPE im, KOMPLEX u)
{
	if(  fabs(fabs(re)-fabs(u.re))<EPSILON &&
		 fabs(fabs(im)-fabs(u.im))<EPSILON &&
		 sqrt((re-u.re)*(re-u.re)+(im-u.im)*(im-u.im)) <EPSILON
	   )
		  return 1;//TRUE
	else return 0;	//FALSE
}


// Identifiziert gleiche Eigenwerte und ermittelt die algebraischen Vielfachheiten
int alg_vielf(BASETYPE *wr,BASETYPE *wi,KOMPLEX *ev,int n,int *av)
/*  wr,wi: Real- und Imaginrteile der von hqr ermittelten Eigenwerte
    ev   : der doublettenfreie Vektor der Eigenwerte 
    av   : Vektor der algebraischen Vielfachheiten in der Reihenfolge entsprechend av
    Returnwert: Anzahl der verschiedenen Eigenwerte	*/
{
	int i,j,r=0,df;
	for(i=1;i<=n;i++)
		{
			j=1;
			df=0; // df=="ist-drin-flag"
			while(j<=r)
				{
					if( gleich(wr[i],wi[i],ev[j])  )
						{
							av[j]++;
							df=1;
						}
					j++;
				}
			if(0==df)
			{
				r++;
				ev[r].re=wr[i];
				ev[r].im=wi[i];
				av[r]=1;
			}
		}
	return r;
}




/* 	Ermittelt die Anzahlen k[i,j], mit der die Jordankstchen J(lbda(i),j) in
	der JNF vorkommen*/
void jblocks(	KOMPLEX **a,// die Ausgangsmatrix
				KOMPLEX *ew,// Vektor der (verschiedenen) Eigenwerte
				int *av,	// Vektor der zugehrigen algebraischen Vielfachheiten
				int r,		// Anzahl der verschiedenen Eigenwerte
				int n,		// Dimension von a
				int **kij)	// Matrix der zu berechnenden k[i,j]
{
	int i,j,m;
	int *s=(int *)malloc((n+1)*sizeof(int));
	for(j=1;j<=r;j++)
		{kij[j][av[j]]=rg_sub_pot(a,ew[j],av[j]-1,n)-n+av[j];
										
			if(kij[j][av[j]]<0)
			{kij[j][av[j]]=0; 
			 printf("\nACHTUNG!: k[%d,%d]=%d < 0. Wurde auf 0 gesetzt.\n",j,av[j],kij[j][av[j]]);}
		}
	for(i=1;i<=r;i++)
		{
			//printf("i:%d\n",i);
			s[av[i]]=0;
			for(j=av[i]-1;j>=1;j--)
				{
					//printf("j:%d\n",j);
					s[j]=s[j+1]+kij[i][j+1];
					for(m=j+1;m<=av[i];m++)
						{s[j]=s[j]+kij[i][m];
						  //printf("m:%d\n",m);
						 }
					kij[i][j]=rg_sub_pot(a,ew[i],j-1,n)-n+av[i]-s[j];
						
						if(kij[i][j]<0)
						{kij[i][j]=0;	
						 printf("\nACHTUNG!: k[%d,%d]=%d < 0. Wurde auf 0 gesetzt.\n",i,j,kij[i][j]);}
				}
		}
	//Probe: Es muss gelten: 
	//	k[i,1]+2k[i,2]+..+av[i]*k[i,av[i]]==av[i]   fuer  i=1..r
		int t;
		for(i=1;i<=r;i++)
			{
				t=0;
				for(j=1;j<=av[i];j++)
					t+=j*kij[i][j];
				if(t!=av[i])
					{
						printf("\nProbe in jblocks negativ.\n");
						//exit(0);
					}
			}
}





void jnf(KOMPLEX **a,KOMPLEX **jf,int n)
{
		
		// Beim gegenwrtigen Stand knnen nur die Eigenwerte von reellen 
		// Matrizen bestimmt werden, also muss die Eingabematrix
		// "reell gemacht" werden
		int i,j,l,m;
		BASETYPE **x=(BASETYPE **)malloc((n+1)*sizeof(BASETYPE *));
				for(i=1;i<=n;i++) x[i]=(BASETYPE *)malloc((n+1)*sizeof(BASETYPE));
	
		for(i=1;i<=n;i++)		
			for(j=1;j<=n;j++)
				{
					 x[i][j]=a[i][j].re;
					 // Ausdrucken zur Kontrolle
					 //printf("%+.5f,",x[i][j]);
					 //if(j%n==0)printf("\n");
				 }
		// jetzt ist x=Re(a)
		
// Eigenwerte ermitteln
		// Zuerst mit den Methoden aus Numerical Recipes
		balanc(x,n);//optional
		elmhes(x,n);// obere Hessenbergform
		BASETYPE *evr= (BASETYPE *)malloc((n+1)*sizeof(BASETYPE));//Eigenwerte: Re
		BASETYPE *evi= (BASETYPE *)malloc((n+1)*sizeof(BASETYPE));//Eigenwerte: Im
		hqr(x,n,evr,evi);// QR-Zerlegung, Eigenwertermittlung
		// Die Real- bzw. Imaginrteile der Eigenwerte sind jetzt in evr bzw. evi
		printf("\nEWe nach Numerical Recipes:\n");
			for(i=1;i<=n;i++)
				printf("  %+14.10f %+.10fI\n",evr[i],evi[i]);


		// Jetzt werden die Eigenwerte mit Methoden aus "mymathlib.c" ermittelt
		// Dort werden Arrays mit 0 beginnend indiziert, so dass einige
		// Hilfsgrssen eingefhrt werden mssen
		// "mm" (wie "mymathlib") markiert diese Grssen
		BASETYPE *mmA  = (BASETYPE*) malloc(sizeof(BASETYPE) *n*n*2);
		BASETYPE **mmpA= (BASETYPE**) malloc(sizeof(BASETYPE) *n*2);
		for(i = 1, *mmpA = mmA; i < n; i++) 
			*(mmpA+i) = *(mmpA + i - 1) + n;		
		for(i=0;i<n;i++)
			for(j=0;j<n;j++)
				mmpA[i][j]=a[i+1][j+1].re;
		// Die Eingabematrix liegt jetzt in mmA
		//BALANCING	
		BASETYPE *mmd =(BASETYPE*) malloc(sizeof(BASETYPE) * n*2);
		int    *mmp =(int*) malloc(sizeof(int) * n*2);
		Balance_Matrix(mmA,mmd,mmp,n);
		// HESSENBERG ORTHO
		BASETYPE   *mmU=(BASETYPE *)malloc(sizeof(BASETYPE)*n*n*2);
		BASETYPE **mmpU=(BASETYPE**) malloc(sizeof(BASETYPE)*n*2);
		for(i = 1, *mmpU = mmU; i < n; i++) 
			*(mmpU+i) = *(mmpU + i - 1) + n;		
		int mmtt=Hessenberg_Form_Orthogonal(mmA,mmU,n);
		// QR
		BASETYPE *mmre=(BASETYPE*) malloc(sizeof(BASETYPE) * n*2);
		BASETYPE *mmim=(BASETYPE*) malloc(sizeof(BASETYPE) * n*2);
		mmtt=QR_Hessenberg_Matrix(mmA,mmU,mmre,mmim,n,30);
		printf("\nEwe nach mymathlib, returns:%d\n",mmtt);
		for(i=0;i<n;i++)
				printf("  %+14.10f %+.10fI\n",mmre[i],mmim[i]);


#ifdef LCC32
		//Jetzt werden die EWe mit den LCC-spezifischen Funktionen ermittelt
		MAT *lccA=m_get(n,n), 
			*lccT,*lccQ;
		VEC *lccre= v_get(n), // Re und Im der Eigenwerte
			*lccim= v_get(n); // evals_re->ve[0] = Re des 1. EW
		for	(i=0;i<n;i++)
			for(j=0;j<n;j++)
				 lccA->me[i][j]=a[i+1][j+1].re;
		lccT = m_copy(lccA,MNULL);
		lccQ=m_get(n,n);
		// compute Schur form: A = Q*T*Q^T 
		schur(lccT,lccQ);// Bringt T in situ auf Schur-Form
				   // auch mglich: T=schur(T,Q);
		// extract eigenvalues 
		schur_evals(lccT,lccre,lccim);
		printf("\nEwe mit lcc:\n");
		for(i=0;i<n;i++)
				printf("  %+14.10f %+.10fI\n",lccre->ve[i],lccim->ve[i]);
#endif

// Eigenwertermittlung abgeschlossen



	printf("\nMit welchen Eigenwerten moechten Sie fortfahren ?");
#ifdef LCC32
	 printf(" (1,2,3):");
#endif


#ifndef LCC32
	 printf(" (1,2):");
#endif
	scanf("%d",&i);


	if(i==2) 
		for(j=0;j<n;j++)
			{evr[j+1]=mmre[j];
			 evi[j+1]=mmim[j];}

#ifdef LCC32
	if(i==3) 
		for(j=0;j<n;j++)
			{evr[j+1]=lccre->ve[j];
			 evi[j+1]=lccim->ve[j];}
#endif

	
	

		// Ermittlung der algebraischen Vielfachheiten
		KOMPLEX *ev= (KOMPLEX *)malloc((n+1)*sizeof(KOMPLEX));// Vektor der 
															   // verschiedenen Eigenwerte
		int 	*av= (int *)malloc((n+1)*sizeof(int));
		int r=alg_vielf(evr,evi,ev,n,av);
		// Die r (verschiedenen) Eigenwerte liegen jetzt in ev, die zugehoerigen 
		// algebraischen Vielfachheiten in av

		printf("\nEWe nach Vielfachheiten:\n");
		for(i=1;i<=r;i++) printf(" ev[%d]=%+9.5f %+9.5fI  Vielf.:%d\n",
							i,ev[i].re,ev[i].im,av[i]);
		
		
		// Ermittlung der der Anzahlen k[i,j], mit der die Jordanblcke der Groesse j zum
		// Eigenwert i vorkommen
		int **k=(int **)malloc((n+1)*sizeof(int *));
		for(i=1;i<=n;i++)
			{
				k[i]=(int *)malloc((n+1)*sizeof(int));
				for(j=1;j<=n;j++)	k[i][j]=0;
			}
		jblocks(a,ev,av,r,n,k);
		// Die k[i,j] liegen jetzt in k
		
		
		// Ausgabe zur Kontrolle / Analyse
		int tt=0;
		for(i=1;i<=r;i++)
			tt+=av[i];
		printf("Summe der alg. Vielfachheiten: %d\n",tt);
		//printf("Anzahl der verschiedenen Eigenwerte: %d\n",r);
		// Notausgang
		if(tt!=n)	{printf("Algebraische Vielfachheiten summieren sich nicht zu n !Abbruch.\n");
					 exit(0);}
		// Ausgabe zur Kontrolle / Analyse
		for(i=1;i<=r;i++)
			for(j=1;j<=av[i];j++)
				printf("k[%d,%d]: %d\n",i,j,k[i][j]);		
	

		
		//Zusammenbauen der JNF
		int zc=1;//"Zeilencount"
		for(i=1;i<=r;i++)
			for(j=1;j<=av[i];j++)
				// Fge k[i,j]-mal einen Jordanblock mit Eigenwert ev[i] der
				//										Grsse j an
				for(l=1;l<=k[i][j];l++)
					for(m=1;m<=j;m++)
						{
							jf[zc][zc]=ev[i];
							if(m!=j) {
									// Notausgang	
									  if(zc+1>n)
										{printf("Unzulaessiger Index bei der JNF !");
										 exit(0);}
									  jf[zc][zc+1].re=1.;
									  jf[zc][zc+1].im=0.;}
							zc++;
						}
		// Die JNF liegt jetzt in jf
}
